program EDDA
! EDDA:integrated simulation of debris flow erosion, deposition and property changes
! by H.X. Chen, HKUST, 2015
use input_file_defs; use input_vars
use grids; use model_vars;use output_file_defs
implicit none
integer::grd
integer:: i,ii,j,k,l,m,n,imx1,mnd,maxzo,maxdirection
integer:: nodata,sctr
integer:: ncol,nrow,u(25)
character (len=255):: outfil 
character (len=14):: header(6)
character (len=8):: date
character (len=10):: time
character (len=4):: stp
character (len=31):: scratch,irfil
character (len=7):: vrsn
character (len=11):: bldate

! first executable statement ............	
     call date_and_time(date,time)
	test=-9999.D0; test1=-9999.
	u=(/11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,&
     &	28,29,30,31,32,33,34,35/)
	pi=3.141592653589793
	eps=1.d-18
	dg2rad=pi/180.D0
	vrsn='1.0'; bldate='14 Jan 2015'
	write (*,*) ''
	write (*,*) 'EDDA:integrated simulation of debris flow erosion,'
	write(*,*) 'deposition and property changes'
	write (*,*) '       Version ', vrsn,', ',bldate
	write (*,*) '  By Hongxin Chen'
	write (*,*) '       The Hong Kong University of Science and Technology'
	write (*,*) '-----------------------------------------'
	
	
! open log file
outfil='EDDALog.txt'
open (u(19),file=trim(outfil),status='unknown',err=410)
	write (u(19),*) ''
     	write (u(19),*) 'Starting EDDA ', vrsn,' ',bldate
     	write (u(19),*) 'Date ',date(5:6),'/',date(7:8),'/',date(1:4)
	write (u(19),*) 'Time ',time(1:2),':',time(3:4),':',time(5:6)
!  read initialization file
call trini(u(19),u(1),dg2rad)

!allocate@iniate arrays for debris flow simulation
grd=row*col
imx1=imax
rhow=1000.
rhos=2650.
allocate (pf1(grd),pf2(grd))
allocate (manning(imax),ele(imax),ltstar(imax),inierodithick(imax),debdepothick(imax),fh(imax),frho(imax),zo(imax),sctrtrace(imax),slo(imax))
allocate (stemp(col),temp(col))
allocate (fp(imax,8),fv(imax,8),maxfv(imax),maxfh(imax))
allocate (ri(imax),outflow(imax))
allocate (tfg(imax))

pf1=0.
pf2=0

ele=0.
manning=0.
ltstar=0.
inierodithick=0.
debdepothick=0.
fh=0.
frho=1000
zo=0
sctrtrace=0
slo=0.
maxfv=0.
maxfh=0.


stemp=0.
temp=0
fp=0
outflow=.false.


! fv(i,1) is the flow velocity cross the northern wall of cell i
! fv(i,2) is the flow velocity cross the northeastern wall of cell i
! fv(i,3) is the flow velocity cross the eastern wall of cell i
! fv(i,4) is the flow velocity cross the southeastern wall of cell i
! fv(i,5) is the flow velocity cross the southern wall of cell i
! fv(i,6) is the flow velocity cross the southwestern wall of cell i
! fv(i,7) is the flow velocity cross the western wall of cell i
! fv(i,8) is the flow velocity cross the northwestern wall of cell i
fv=0.

! rideb is the rainfall intensity of each cell during each rainfall interval for debris flow simulation
allocate (rideb(imax,nper))
rideb=0.

! *****************************************************************
!  read gridded data from GIS
write (*,*) 'Reading input grids'
write(u(19),*) 'Input file name,            Cell count'

! read dem file
! ele(i) is the surface elevation for cell i that is not erodible   
call srdgrd(grd,col,ncol,nrow,celsiz,nodat,&
          &ele,pf1,sctr,imax,stemp,u(8),demfil,param,header,u(19))
    	  write(u(19),*) 'Dem file  grid'
     	  write(u(19),*) trim(demfil),sctr,' data cells'
     	  if(sctr/=imx1) write (u(19),*) 'Grid mismatch ',trim(demfil)
! read slope file
call srdgrd(grd,col,ncol,nrow,celsiz,nodat,&
          &slo,pf1,sctr,imax,stemp,u(8),slofil,param,header,u(19))
    	  write(u(19),*) 'Slope file  grid'
     	  write(u(19),*) trim(slofil),sctr,' data cells'
     	  if(sctr/=imx1) write (u(19),*) 'Grid mismatch ',trim(slofil) 	  
slo=slo*dg2rad
     	  
 ! read ltstar file
 if (cltstar>0.) then
 ltstar=cltstar
 else
call srdgrd(grd,col,ncol,nrow,celsiz,nodat,&
          &ltstar,pf1,sctr,imax,stemp,u(8),ltstarfil,param,header,u(19))
    	  write(u(19),*) 'Soil thickness file  grid'
     	  write(u(19),*) trim(ltstarfil),sctr,' data cells'
     	  if(sctr/=imx1) write (u(19),*) 'Grid mismatch ',trim(ltstarfil)     	
 end if  
 ! inierodithick(i) is the initial erodible thickness for cell i
 inierodithick=ltstar
     	  
! read manning coefficient
! manning(i) is the Manning's coefficient for cell i
if (cmanning>=0) then
manning=cmanning
else
call srdgrd(grd,col,ncol,nrow,celsiz,nodat,&
          &manning,pf1,sctr,imax,stemp,u(8),manningfil,param,header,u(19))
    	  write(u(19),*) 'Manning file  grid'
     	  write(u(19),*) trim(manningfil),sctr,' data cells'
     	  if(sctr/=imx1) write (u(19),*) 'Grid mismatch ',trim(manningfil)
end if

! read the zone file
if(nzon==1) then
	  zo=1 ! if only one zone, all values of zone grid equal 1.
	  write(*,*) 'One property zone, no grid required!'
	  write(u(19),*) 'One property zone, no grid required!'
	  parami=param ! added 7/29/2008 RLB
	else
	  call irdgrd(grd,col,ncol,nrow,celsiz,nodata,mnd,&
          &zo,pf2,sctrtrace,sctr,imax,temp,u(15),zonfil,parami,header,u(19))
    	  write(u(19),*) 'Property zone grid'
      	  write(u(19),*) trim(zonfil),sctr,' data cells'
     	  if(sctr/=imx1) then
     	    write (*,*) 'Grid mismatch ',trim(zonfil)
     	    write (*,*) 'Correct property-zone grid and/or initializtion file.' 
     	    write (u(19),*) 'Grid mismatch ',trim(zonfil)
     	    write (u(19),*) 'Correct property-zone grid and/or initializtion file.' 
     	    close(u(19))
     	    pause 'Press return/enter to quit'
     	    stop '-1'
     	  end if
     	  maxzo=maxval(zo)

     	  if (maxzo/=nzon) then
     	    write (*,*) 'Maximum zone number does not equal number of property zones!'
     	    write (*,*) 'Correct property-zone grid and/or initializtion file.' 
     	    write (u(19),*) 'Maximum zone number does not equal number of property zones!'
     	    write (u(19),*) 'Correct property-zone grid and/or initializtion file.' 
     	    close(u(19))
     	    pause 'Press return/enter to quit'
     	    stop '-1'
     	  end if
	end if

! for four flow direction system
! fp(i,1) is the northern nodal point of cell i
! fp(i,2) is the eastern nodal point of cell i
! fp(i,3) is the southern nodal point of cell i
! fp(i,4) is the western nodal point of cell i

! for eight flow direction system
! fp(i,1) is the northern nodal point of cell i
! fp(i,2) is the northeastern nodal point of cell i
! fp(i,3) is the eastern nodal point of cell i
! fp(i,4) is the southeastern nodal point of cell i
! fp(i,5) is the southern nodal point of cell i
! fp(i,6) is the southwestern nodal point of cell i
! fp(i,7) is the western nodal point of cell i
! fp(i,8) is the northwestern nodal point of cell i

call flodir(grd,col,ncol,nrow,celsiz,nodat,&
          &ele,pf1,sctr,imax,stemp,u(8),demfil,param,header,u(19),fp,outflow)
 maxdirection=size(fp(1,:))  
    
 ! judge whether the cell is the outflow cell
 do i=1,imx1
 do ii=1,maxdirection
 if (fp(i,ii)==0) then
 outflow(i)=.true.
 exit
 end if
 end do
 end do
	                     
! read rainfall information
do j=1,nper
  	  if (cri(j).lt.0) then
	  call srdgrd(grd,col,ncol,nrow,celsiz,nodat,ri,&
     	  &pf1,sctr,imax,stemp,u(13),rifil(j),param,header,u(19))
    	  write(u(19),*) 'Precipitation intensity grid ',j
     	  write(u(19),*) trim(rifil(j)),sctr,' data cells'
     	  if(sctr/=imx1) write (u(19),*) 'Grid mismatch ',trim(rifil(j))
     	  
      else
	  do  i=1,imx1
	   ri(i)=cri(j)
  	  end do
  	 end if
  	 rideb(:,j)=ri(:)
  	 
 ! end do j
 end do

! read information of inflow hydrographs
if (inflowsimul) call inflow_read(u(19),u(2))


! initialize time parameters
! t0 is the starting time of simulation
! dt is the time step of simulation, which is set to be dtmin at the beginning
! tout is the output interval, ttout is the output time
! tminimum is used to record the minimum time step
! tmaximum is used to record the maximum time step
! tnow is the present time
! ntsdeb is used to record the number of time step of debris flow simulation
t0=0.
dt=dtmin
ttout=tout
tminimum=99.
tmaximum=-99
tnow=t0
ntsdeb=0

! compute the maximum possbile steps to finish the simulation
maxnts=2*simul/dtmin

call dfs(imx1,nrow,ncol,header,u,maxdirection)

write (*,*) 'The total number of times is',' ',ntsdeb
avedt=tnow/float(ntsdeb)
write (*,*) 'The average time step interval is',' ',avedt
write (u(19),*) 'The total number of times is',' ',ntsdeb
write (u(19),*) 'The average time step interval is',' ',avedt
close (u(19))

pause 'The program EDDA runs successfully!'
stop '0'
! Error reporting 	
  410	continue
  	write (*,*) 'Error opening output file main program 410'
	write (*,*) '--> ',outfil
	write (*,*) 'Check file path and status'
  	write (u(19),*) 'Error opening output file main program 410'
	write (u(19),*) '--> ',outfil
	write (u(19),*) 'Check file path and status'
	pause 'Press RETURN to exit'
	stop '410'

end